<?php
/**
 * Class for providing helper methods for omnipress.
 *
 * @package Omnipress
 */

namespace Omnipress;

/**
 * Class for providing helper methods for omnipress.
 *
 * @since 1.1.0
 */
class Helpers {

	/**
	 * Returns plugins status.
	 *
	 * @param string $plugin_slug
	 * @return string active | paused | not-found
	 */
	public static function get_plugin_status( $plugin_slug ) {

		if ( ! function_exists( 'is_plugin_active' ) ) {
			require_once ABSPATH . 'wp-admin/includes/plugin.php';
		}

		if ( is_plugin_active( $plugin_slug ) ) {
			return 'active';
		}

		static $plugins = array();

		if ( ! $plugins ) {
			$plugins = get_plugins();
		}

		if ( isset( $plugins[ $plugin_slug ] ) ) {
			return 'paused';
		}

		return 'not-found';

	}

	/**
	 * Extract image urls from the content strings.
	 *
	 * @param string $content
	 * @return array
	 */
	public static function extract_img_urls( $content ) {
		preg_match_all( '~<img.*?src=["\']+(.*?)["\']+~', $content, $urls );

		if ( empty( $urls[1] ) ) {
			return array();
		}

		return $urls[1];
	}

	/**
	 * Downloads images and returns their respective local urls
	 * with key being source urls and value being local urls.
	 *
	 * @return array
	 */
	public static function download_images( $img_urls ) {

		if ( ! $img_urls ) {
			return array();
		}

		$downloaded_images_urls = get_option( 'omnipress_downloaded_images_urls', array() );

		if ( is_array( $img_urls ) && ! empty( $img_urls ) ) {

			if ( ! function_exists( 'download_url' ) ) {
				require_once ABSPATH . 'wp-admin/includes/file.php';
			}

			if ( ! function_exists( 'media_sideload_image' ) ) {
				require_once ABSPATH . 'wp-admin/includes/media.php';
			}

			if ( ! function_exists( 'wp_read_image_metadata' ) ) {
				require_once ABSPATH . 'wp-admin/includes/image.php';
			}

			$site_url = site_url();

			foreach ( $img_urls as $img_url ) {

				$source_url = wp_normalize_path( trim( $img_url ) );

				if ( isset( $downloaded_images_urls[ $source_url ] ) ) {

					$url_to_path = wp_normalize_path( str_replace( $site_url, ABSPATH, $downloaded_images_urls[ $source_url ] ) );

					if ( file_exists( $url_to_path ) ) {

						/**
						 * Ignore if downloaded already.
						 */
						continue;
					}
				}

				$src = media_sideload_image( $source_url, 0, null, 'src' );

				if ( ! is_wp_error( $src ) ) {
					$downloaded_images_urls[ $source_url ] = $src;
				}
			}
		}

		update_option( 'omnipress_downloaded_images_urls', $downloaded_images_urls );

		return $downloaded_images_urls;

	}
}